@@ -3,8 +3,8 @@ from django.db import models
 from drf_spectacular.drainage import add_trace_message, get_override, has_override, warn
 from drf_spectacular.extensions import OpenApiFilterExtension
 from drf_spectacular.plumbing import (
-    build_array_type, build_basic_type, build_parameter_type, follow_field_source, get_type_hints,
-    get_view_model, is_basic_type,
+    build_array_type, build_basic_type, build_parameter_type, follow_field_source, get_manager,
+    get_type_hints, get_view_model, is_basic_type,
 )
 from drf_spectacular.types import OpenApiTypes
 from drf_spectacular.utils import OpenApiParameter
@@ -52,12 +52,12 @@ class DjangoFilterExtension(OpenApiFilterExtension):
         if not model:
             return []
 
-        filterset_class = self.target.get_filterset_class(auto_schema.view, model.objects.none())
+        filterset_class = self.target.get_filterset_class(auto_schema.view, get_manager(model).none())
         if not filterset_class:
             return []
 
         result = []
-        with add_trace_message(filterset_class.__name__):
+        with add_trace_message(filterset_class):
             for field_name, filter_field in filterset_class.base_filters.items():
                 result += self.resolve_filter_field(
                     auto_schema, model, filterset_class, field_name, filter_field
@@ -84,8 +84,11 @@ class DjangoFilterExtension(OpenApiFilterExtension):
         }
         filter_method = self._get_filter_method(filterset_class, filter_field)
         filter_method_hint = self._get_filter_method_hint(filter_method)
+        filter_choices = self._get_explicit_filter_choices(filter_field)
+        schema_from_override = False
 
         if has_override(filter_field, 'field') or has_override(filter_method, 'field'):
+            schema_from_override = True
             annotation = (
                 get_override(filter_field, 'field') or get_override(filter_method, 'field')
             )
@@ -116,17 +119,24 @@ class DjangoFilterExtension(OpenApiFilterExtension):
                 schema = build_basic_type(OpenApiTypes.NUMBER)  # TODO may be improved
             else:
                 schema = build_basic_type(OpenApiTypes.NUMBER)
-        else:
+        elif isinstance(filter_field, (filters.ChoiceFilter, filters.MultipleChoiceFilter)):
             try:
-                # the last resort is to lookup the type via the model or queryset field.
-                # first search for the field in the model as this has the least amount of
-                # potential side effects. Only after that fails, attempt to call
-                # get_queryset() to check for potential query annotations.
-                model_field = self._get_model_field(filter_field, model)
-                if not isinstance(model_field, models.Field):
-                    qs = auto_schema.view.get_queryset()
-                    model_field = qs.query.annotations[filter_field.field_name].field
-                schema = auto_schema._map_model_field(model_field, direction=None)
+                schema = self._get_schema_from_model_field(auto_schema, filter_field, model)
+            except Exception:
+                if filter_choices and is_basic_type(type(filter_choices[0])):
+                    # fallback to type guessing from first choice element
+                    schema = build_basic_type(type(filter_choices[0]))
+                else:
+                    warn(
+                        f'Unable to guess choice types from values, filter method\'s type hint '
+                        f'or find "{field_name}" in model. Defaulting to string.'
+                    )
+                    schema = build_basic_type(OpenApiTypes.STR)
+        else:
+            # the last resort is to look up the type via the model or queryset field
+            # and emit a warning if we were unsuccessful.
+            try:
+                schema = self._get_schema_from_model_field(auto_schema, filter_field, model)
             except Exception as exc:  # pragma: no cover
                 warn(
                     f'Exception raised while trying resolve model field for django-filter '
@@ -139,8 +149,9 @@ class DjangoFilterExtension(OpenApiFilterExtension):
         schema.pop('readOnly', None)
         # enrich schema with additional info from filter_field
         enum = schema.pop('enum', None)
-        if 'choices' in filter_field.extra:
-            enum = [c for c, _ in filter_field.extra['choices']]
+        # explicit filter choices may disable enum retrieved from model
+        if not schema_from_override and filter_choices is not None:
+            enum = filter_choices
         if enum:
             schema['enum'] = sorted(enum, key=str)
 
@@ -203,8 +214,50 @@ class DjangoFilterExtension(OpenApiFilterExtension):
         except:  # noqa: E722
             return _NoHint
 
+    def _get_explicit_filter_choices(self, filter_field):
+        if 'choices' not in filter_field.extra:
+            return None
+        elif callable(filter_field.extra['choices']):
+            # choices function may utilize the DB, so refrain from actually calling it.
+            return []
+        else:
+            return [c for c, _ in filter_field.extra['choices']]
+
     def _get_model_field(self, filter_field, model):
         if not filter_field.field_name:
             return None
         path = filter_field.field_name.split('__')
         return follow_field_source(model, path, emit_warnings=False)
+
+    def _get_schema_from_model_field(self, auto_schema, filter_field, model):
+        # Has potential to throw exceptions. Needs to be wrapped in try/except!
+        #
+        # first search for the field in the model as this has the least amount of
+        # potential side effects. Only after that fails, attempt to call
+        # get_queryset() to check for potential query annotations.
+        model_field = self._get_model_field(filter_field, model)
+
+        # this is a cross feature between rest-framework-gis and django-filter. Regular
+        # behavior needs to be sidestepped as the model information is lost down the line.
+        # TODO for now this will be just a string to cover WKT, WKB, and urlencoded GeoJSON
+        #   build_geo_schema(model_field) would yield the correct result
+        if self._is_gis(model_field):
+            return build_basic_type(OpenApiTypes.STR)
+
+        if not isinstance(model_field, models.Field):
+            qs = auto_schema.view.get_queryset()
+            model_field = qs.query.annotations[filter_field.field_name].field
+        return auto_schema._map_model_field(model_field, direction=None)
+
+    @classmethod
+    def _is_gis(cls, field):
+        if not getattr(cls, '_has_gis', True):
+            return False
+        try:
+            from django.contrib.gis.db.models import GeometryField
+            from rest_framework_gis.filters import GeometryFilter
+
+            return isinstance(field, (GeometryField, GeometryFilter))
+        except: # noqa
+            cls._has_gis = False
+            return False
