@@ -1,6 +1,6 @@
 import inspect
 import sys
-from typing import Any, Callable, Dict, List, Optional, Tuple, Type, TypeVar, Union
+from typing import Any, Callable, Dict, Optional, Sequence, Tuple, Type, TypeVar, Union
 
 # direct import due to https://github.com/microsoft/pyright/issues/3025
 if sys.version_info >= (3, 8):
@@ -44,33 +44,49 @@ class PolymorphicProxySerializer(Serializer):
             return Response(...)
 
     **Beware** that this is not a real serializer and it will raise an AssertionError
-    if used in that way. It **cannot** be used in views as `serializer_class`
+    if used in that way. It **cannot** be used in views as ``serializer_class``
     or as field in an actual serializer. It is solely meant for annotation purposes.
 
     Also make sure that each sub-serializer has a field named after the value of
     ``resource_type_field_name`` (discriminator field). Generated clients will likely
-    depend on the existence of this field. Setting ``resource_type_field_name``
-    to ``None`` will remove the discriminator altogether. This may be useful in
-    certain situations, but will most likely break client generation.
+    depend on the existence of this field.
+
+    Setting ``resource_type_field_name`` to ``None`` will remove the discriminator
+    altogether. This may be useful in certain situations, but will most likely break
+    client generation. Another use-case is explicit control over sub-serializer's ``many``
+    attribute. To explicitly control this aspect, you need disable the discriminator with
+    ``resource_type_field_name=None`` as well as disable automatic list handling with
+    ``many=False``.
 
     It is **strongly** recommended to pass the ``Serializers`` as **list**,
     and by that let *drf-spectacular* retrieve the field and handle the mapping
     automatically. In special circumstances, the field may not available when
-    drf-spectacular processes the serializer. In those cases you can explicitly state
+    *drf-spectacular* processes the serializer. In those cases you can explicitly state
     the mapping with ``{'legal': LegalPersonSerializer, ...}``, but it is then your
     responsibility to have a valid mapping.
     """
     def __init__(
             self,
             component_name: str,
-            serializers: Union[List[_SerializerType], Dict[str, _SerializerType]],
+            serializers: Union[Sequence[_SerializerType], Dict[str, _SerializerType]],
             resource_type_field_name: Optional[str],
-            many: bool = False,
+            many: Optional[bool] = None,
     ):
         self.component_name = component_name
         self.serializers = serializers
         self.resource_type_field_name = resource_type_field_name
-        super().__init__(many=many)
+        if self._many is False:  # type: ignore[attr-defined]
+            set_override(self, 'many', False)
+        super().__init__()
+
+    def __new__(cls, *args, **kwargs):
+        many = kwargs.pop('many', None)
+        if many is True:
+            instance = cls.many_init(*args, **kwargs)
+        else:
+            instance = super().__new__(cls, *args, **kwargs)
+        instance._many = many
+        return instance
 
     @property
     def data(self):
@@ -98,8 +114,11 @@ class OpenApiExample(OpenApiSchemaBase):
     Helper class to document a API parameter / request body / response body
     with a concrete example value.
 
+    It is recommended to provide a singular example value, since pagination
+    and list responses are handled by drf-spectacular.
+
     The example will be attached to the operation object where appropriate,
-    i. e. where the given ``media_type``, ``status_code`` and modifiers match.
+    i.e. where the given ``media_type``, ``status_code`` and modifiers match.
     Example that do not match any scenario are ignored.
     """
     def __init__(
@@ -113,7 +132,7 @@ class OpenApiExample(OpenApiSchemaBase):
             response_only: bool = False,
             parameter_only: Optional[Tuple[str, _ParameterLocationType]] = None,
             media_type: str = 'application/json',
-            status_codes: Optional[List[str]] = None,
+            status_codes: Optional[Sequence[Union[str, int]]] = None,
     ):
         self.name = name
         self.summary = summary
@@ -124,7 +143,7 @@ class OpenApiExample(OpenApiSchemaBase):
         self.response_only = response_only
         self.parameter_only = parameter_only
         self.media_type = media_type
-        self.status_codes = status_codes or ['200', '201']
+        self.status_codes = list(map(str, status_codes)) if status_codes else ['200', '201']
 
 
 class OpenApiParameter(OpenApiSchemaBase):
@@ -150,16 +169,18 @@ class OpenApiParameter(OpenApiSchemaBase):
             location: _ParameterLocationType = QUERY,
             required: bool = False,
             description: str = '',
-            enum: Optional[List[Any]] = None,
+            enum: Optional[Sequence[Any]] = None,
+            pattern: Optional[str] = None,
             deprecated: bool = False,
             style: Optional[str] = None,
             explode: Optional[bool] = None,
             default: Any = None,
             allow_blank: bool = True,
-            examples: Optional[List[OpenApiExample]] = None,
+            many: Optional[bool] = None,
+            examples: Optional[Sequence[OpenApiExample]] = None,
             extensions: Optional[Dict[str, Any]] = None,
             exclude: bool = False,
-            response: Union[bool, List[Union[int, str]]] = False,
+            response: Union[bool, Sequence[Union[int, str]]] = False,
     ):
         self.name = name
         self.type = type
@@ -167,11 +188,13 @@ class OpenApiParameter(OpenApiSchemaBase):
         self.required = required
         self.description = description
         self.enum = enum
+        self.pattern = pattern
         self.deprecated = deprecated
         self.style = style
         self.explode = explode
         self.default = default
         self.allow_blank = allow_blank
+        self.many = many
         self.examples = examples or []
         self.extensions = extensions
         self.exclude = exclude
@@ -191,7 +214,7 @@ class OpenApiResponse(OpenApiSchemaBase):
             self,
             response: Any = None,
             description: str = '',
-            examples: Optional[List[OpenApiExample]] = None
+            examples: Optional[Sequence[OpenApiExample]] = None
     ):
         self.response = response
         self.description = description
@@ -233,22 +256,22 @@ class OpenApiCallback(OpenApiSchemaBase):
 
 def extend_schema(
         operation_id: Optional[str] = None,
-        parameters: Optional[List[Union[OpenApiParameter, _SerializerType]]] = None,
+        parameters: Optional[Sequence[Union[OpenApiParameter, _SerializerType]]] = None,
         request: Any = empty,
         responses: Any = empty,
-        auth: Optional[List[str]] = None,
+        auth: Optional[Sequence[str]] = None,
         description: Optional[str] = None,
         summary: Optional[str] = None,
         deprecated: Optional[bool] = None,
-        tags: Optional[List[str]] = None,
+        tags: Optional[Sequence[str]] = None,
         filters: Optional[bool] = None,
         exclude: bool = False,
         operation: Optional[Dict] = None,
-        methods: Optional[List[str]] = None,
-        versions: Optional[List[str]] = None,
-        examples: Optional[List[OpenApiExample]] = None,
+        methods: Optional[Sequence[str]] = None,
+        versions: Optional[Sequence[str]] = None,
+        examples: Optional[Sequence[OpenApiExample]] = None,
         extensions: Optional[Dict[str, Any]] = None,
-        callbacks: Optional[List[OpenApiCallback]] = None,
+        callbacks: Optional[Sequence[OpenApiCallback]] = None,
         external_docs: Optional[Union[Dict[str, str], str]] = None,
 ) -> Callable[[F], F]:
     """
@@ -415,7 +438,8 @@ def extend_schema(
             if operation_id is not None or operation is not None:
                 error(
                     f'using @extend_schema on viewset class {f.__name__} with parameters '
-                    f'operation_id or operation will most likely result in a broken schema.'
+                    f'operation_id or operation will most likely result in a broken schema.',
+                    delayed=f,
                 )
             # reorder schema class MRO so that view method annotation takes precedence
             # over view class annotation. only relevant if there is a method annotation
@@ -481,9 +505,9 @@ def extend_schema_field(
 
 def extend_schema_serializer(
         many: Optional[bool] = None,
-        exclude_fields: Optional[List[str]] = None,
-        deprecate_fields: Optional[List[str]] = None,
-        examples: Optional[List[OpenApiExample]] = None,
+        exclude_fields: Optional[Sequence[str]] = None,
+        deprecate_fields: Optional[Sequence[str]] = None,
+        examples: Optional[Sequence[OpenApiExample]] = None,
         extensions: Optional[Dict[str, Any]] = None,
         component_name: Optional[str] = None,
 ) -> Callable[[F], F]:
@@ -543,7 +567,8 @@ def extend_schema_view(**kwargs) -> Callable[[F], F]:
             if method_name not in available_view_methods:
                 warn(
                     f'@extend_schema_view argument "{method_name}" was not found on view '
-                    f'{view.__name__}. method override for "{method_name}" will be ignored.'
+                    f'{view.__name__}. method override for "{method_name}" will be ignored.',
+                    delayed=view
                 )
                 continue
 
