@@ -14,8 +14,8 @@ from tests.models import SimpleModel, SimpleSerializer
 
 try:
     from django_filters.rest_framework import (
-        AllValuesFilter, BaseInFilter, BooleanFilter, CharFilter, DjangoFilterBackend, FilterSet,
-        ModelChoiceFilter, ModelMultipleChoiceFilter, MultipleChoiceFilter, NumberFilter,
+        AllValuesFilter, BaseInFilter, BooleanFilter, CharFilter, ChoiceFilter, DjangoFilterBackend,
+        FilterSet, ModelChoiceFilter, ModelMultipleChoiceFilter, MultipleChoiceFilter, NumberFilter,
         NumericRangeFilter, OrderingFilter, RangeFilter, UUIDFilter,
     )
 except ImportError:
@@ -30,6 +30,7 @@ except ImportError:
             pass
 
     CharFilter = NumberFilter
+    ChoiceFilter = NumberFilter
     OrderingFilter = NumberFilter
     BaseInFilter = NumberFilter
     BooleanFilter = NumberFilter
@@ -117,6 +118,23 @@ class ProductFilter(FilterSet):
         RangeFilter(field_name='price_vat')
     )
 
+    def get_choices(*args, **kwargs):
+        return (('A', 'aaa'),)
+
+    cat_callable = ChoiceFilter(field_name="category", choices=get_choices)
+
+    choice_field_enum_override = extend_schema_field(OpenApiTypes.STR)(ChoiceFilter(choices=(('A', 'aaa'), )))
+
+    # will guess type from choices as a last resort
+    untyped_choice_field_method_with_explicit_choices = ChoiceFilter(
+        method="filter_method_untyped",
+        choices=[(1, 'one')],
+    )
+    untyped_multiple_choice_field_method_with_explicit_choices = MultipleChoiceFilter(
+        method="filter_method_multi_untyped",
+        choices=[(1, 'one')],
+    )
+
     class Meta:
         model = Product
         fields = [
@@ -130,6 +148,9 @@ class ProductFilter(FilterSet):
     def filter_method_untyped(self, queryset, name, value):
         return queryset.filter(id=int(value))  # pragma: no cover
 
+    def filter_method_multi_untyped(self, queryset, name, value):
+        return queryset.filter(id__in=int(value))  # pragma: no cover
+
     # email makes no sense here. it's just to test decoration
     @extend_schema_field(OpenApiTypes.EMAIL)
     def filter_method_decorated(self, queryset, name, value):
@@ -218,6 +239,9 @@ def test_django_filters_requests(no_warnings):
     response = APIClient().get('/api/products/?multi_cat=A&multi_cat=B')
     assert response.status_code == 200, response.content
     assert len(response.json()) == 1
+    response = APIClient().get('/api/products/?cat_callable=A')
+    assert response.status_code == 200, response.content
+    assert len(response.json()) == 1
 
 
 @pytest.mark.contrib('django_filter')
