@@ -4,8 +4,10 @@ from unittest import mock
 
 import pytest
 from django.urls import include, path
+from rest_framework import viewsets
 
 from tests import assert_schema, generate_schema
+from tests.models import SimpleModel, SimpleSerializer
 
 transforms = [
     # User model first_name differences
@@ -45,3 +47,23 @@ def test_rest_auth_token(no_warnings, settings):
     assert_schema(
         schema, 'tests/contrib/test_rest_auth_token.yml', transforms=transforms
     )
+
+
+@pytest.mark.contrib('dj_rest_auth', 'rest_framework_simplejwt')
+@mock.patch('django.conf.settings.JWT_AUTH_COOKIE', 'jwt-session', create=True)
+def test_rest_auth_simplejwt_cookie(no_warnings):
+    from dj_rest_auth.jwt_auth import JWTCookieAuthentication
+
+    class XViewset(viewsets.ModelViewSet):
+        serializer_class = SimpleSerializer
+        queryset = SimpleModel.objects.all()
+        authentication_classes = [JWTCookieAuthentication]
+
+    schema = generate_schema('/x', XViewset)
+    assert schema['paths']['/x/']['get']['security'] == [
+        {'jwtHeaderAuth': []}, {'jwtCookieAuth': []}, {}
+    ]
+    assert schema['components']['securitySchemes'] == {
+        'jwtCookieAuth': {'type': 'apiKey', 'in': 'cookie', 'name': 'jwt-session'},
+        'jwtHeaderAuth': {'type': 'http', 'scheme': 'bearer', 'bearerFormat': 'JWT'}
+    }
